@@ -1,7 +1,6 @@
 /* Miscellaneous functions, not really specific to GNU tar.
 
-   Copyright 1988, 1992, 1994-1997, 1999-2001, 2003-2007, 2009-2010,
-   2012-2014 Free Software Foundation, Inc.
+   Copyright 1988-2019 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,8 +28,15 @@
 # define DOUBLE_SLASH_IS_DISTINCT_ROOT 0
 #endif
 
+static void namebuf_add_dir (namebuf_t, char const *);
+static char *namebuf_finish (namebuf_t);
 static const char *tar_getcdpath (int);
 
+char const *
+quote_n_colon (int n, char const *arg)
+{
+  return quotearg_n_style_colon (n, get_quoting_style (NULL), arg);
+}
 
 /* Handling strings.  */
 
@@ -43,6 +49,20 @@ assign_string (char **string, const char *value)
   *string = value ? xstrdup (value) : 0;
 }
 
+void
+assign_string_n (char **string, const char *value, size_t n)
+{
+  free (*string);
+  if (value)
+    {
+      size_t l = strnlen (value, n);
+      char *p = xmalloc (l + 1);
+      memcpy (p, value, l);
+      p[l] = 0;
+      *string = p;
+    }
+}
+
 #if 0
 /* This function is currently unused; perhaps it should be removed?  */
 
@@ -294,8 +314,6 @@ normalize_filename (int cdidx, const char *name)
       size_t copylen;
       bool need_separator;
 
-      if (!cdpath)
-	call_arg_fatal ("getcwd", ".");
       copylen = strlen (cdpath);
       need_separator = ! (DOUBLE_SLASH_IS_DISTINCT_ROOT
 			  && copylen == 2 && ISSLASH (cdpath[1]));
@@ -586,7 +604,12 @@ safer_rmdir (const char *file_name)
       return -1;
     }
 
-  return unlinkat (chdir_fd, file_name, AT_REMOVEDIR);
+  if (unlinkat (chdir_fd, file_name, AT_REMOVEDIR) == 0)
+    {
+      remove_delayed_set_stat (file_name);
+      return 0;
+    }
+  return -1;
 }
 
 /* Remove FILE_NAME, returning 1 on success.  If FILE_NAME is a directory,
@@ -650,7 +673,7 @@ remove_any_file (const char *file_name, enum remove_option option)
 		 (entrylen = strlen (entry)) != 0;
 		 entry += entrylen + 1)
 	      {
-		char *file_name_buffer = new_name (file_name, entry);
+		char *file_name_buffer = make_file_name (file_name, entry);
 		int r = remove_any_file (file_name_buffer,
                                          RECURSIVE_REMOVE_OPTION);
 		int e = errno;
@@ -715,7 +738,7 @@ maybe_backup_file (const char *file_name, bool this_is_the_archive)
       && (S_ISBLK (file_stat.st_mode) || S_ISCHR (file_stat.st_mode)))
     return true;
 
-  after_backup_name = find_backup_file_name (file_name, backup_type);
+  after_backup_name = find_backup_file_name (chdir_fd, file_name, backup_type);
   if (! after_backup_name)
     xalloc_die ();
 
@@ -897,6 +920,8 @@ chdir_arg (char const *dir)
 	{
 	  wd[wd_count].name = ".";
 	  wd[wd_count].abspath = xgetcwd ();
+	  if (!wd[wd_count].abspath)
+	    call_arg_fatal ("getcwd", ".");
 	  wd[wd_count].fd = AT_FDCWD;
 	  wd_count++;
 	}
@@ -1022,7 +1047,11 @@ tar_getcdpath (int idx)
     {
       static char *cwd;
       if (!cwd)
-	cwd = xgetcwd ();
+	{
+	  cwd = xgetcwd ();
+	  if (!cwd)
+	    call_arg_fatal ("getcwd", ".");
+	}
       return cwd;
     }
   return wd[idx].abspath;
@@ -1032,7 +1061,10 @@ void
 close_diag (char const *name)
 {
   if (ignore_failed_read_option)
-    close_warn (name);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	close_warn (name);
+    }
   else
     close_error (name);
 }
@@ -1041,7 +1073,10 @@ void
 open_diag (char const *name)
 {
   if (ignore_failed_read_option)
-    open_warn (name);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	open_warn (name);
+    }
   else
     open_error (name);
 }
@@ -1050,7 +1085,10 @@ void
 read_diag_details (char const *name, off_t offset, size_t size)
 {
   if (ignore_failed_read_option)
-    read_warn_details (name, offset, size);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	read_warn_details (name, offset, size);
+    }
   else
     read_error_details (name, offset, size);
 }
@@ -1059,7 +1097,10 @@ void
 readlink_diag (char const *name)
 {
   if (ignore_failed_read_option)
-    readlink_warn (name);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	readlink_warn (name);
+    }
   else
     readlink_error (name);
 }
@@ -1068,7 +1109,10 @@ void
 savedir_diag (char const *name)
 {
   if (ignore_failed_read_option)
-    savedir_warn (name);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	savedir_warn (name);
+    }
   else
     savedir_error (name);
 }
@@ -1077,7 +1121,10 @@ void
 seek_diag_details (char const *name, off_t offset)
 {
   if (ignore_failed_read_option)
-    seek_warn_details (name, offset);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	seek_warn_details (name, offset);
+    }
   else
     seek_error_details (name, offset);
 }
@@ -1086,7 +1133,10 @@ void
 stat_diag (char const *name)
 {
   if (ignore_failed_read_option)
-    stat_warn (name);
+    {
+      if (WARNING_ENABLED(WARN_FAILED_READ))
+	stat_warn (name);
+    }
   else
     stat_error (name);
 }
@@ -1106,13 +1156,6 @@ file_removed_diag (const char *name, bool top_level,
     diagfn (name);
 }
 
-void
-write_fatal_details (char const *name, ssize_t status, size_t size)
-{
-  write_error_details (name, status, size);
-  fatal_exit ();
-}
-
 /* Fork, aborting if unsuccessful.  */
 pid_t
 xfork (void)
@@ -1197,7 +1240,7 @@ namebuf_name (namebuf_t buf, const char *name)
   return buf->buffer;
 }
 
-void
+static void
 namebuf_add_dir (namebuf_t buf, const char *name)
 {
   static char dirsep[] = { DIRECTORY_SEPARATOR, 0 };
@@ -1210,7 +1253,7 @@ namebuf_add_dir (namebuf_t buf, const char *name)
   buf->dir_length += strlen (name);
 }
 
-char *
+static char *
 namebuf_finish (namebuf_t buf)
 {
   char *res = buf->buffer;
